package com.blackledge.david.html.hme;

import java.awt.Color;

import com.tivo.hme.bananas.BHighlights;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.bananas.BSkin.Element;
import com.tivo.hme.sdk.Resource;

public class BScrollBar extends BView {
	private static int pagerHeight = 20;
	private static int scrollerWidth = 20;
	private BView scroller;
	private BView pageUp;
	private BView pageDown;
	private int scrollMaxHeight = -1;
	private int width;
	
	public BScrollBar(BView targetArea, BView parent) {
		this(targetArea, parent, -1);
	}
	
	public BScrollBar(BView targetArea, BView parent, int width) {
		super(parent, targetArea.getWidth()+targetArea.getX(), targetArea.getY()+pagerHeight,
				width>0?width:scrollerWidth, targetArea.getHeight()-pagerHeight*2);
		
		this.width = width;
		BScrollBar scrollBase = this;
		scrollBase.setResource(Color.DARK_GRAY);
		
		scroller = new BView(scrollBase, 0, 0, scrollBase.getWidth(), scrollBase.getHeight());
		Element barSkin = getBApp().getSkin().get(H_BAR);
		scroller.setResource(barSkin.getResource(), RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER | RSRC_IMAGE_VFIT);
		
		 pageUp = new BView(parent,
				scrollBase.getX(), scrollBase.getY()-pagerHeight,
				getScrollerWidth(), pagerHeight);
		BHighlights pageUpHighlight = pageUp.getHighlights();
		pageUpHighlight.setPageHint(H_PAGEUP, A_CENTER, A_BOTTOM);
		
		pageDown = new BView(parent,
				scrollBase.getX(), scrollBase.getY()
						+ scrollBase.getHeight(), getScrollerWidth(), pagerHeight);
		BHighlights pageDownHighlight = pageDown.getHighlights();
		pageDownHighlight.setPageHint(H_PAGEDOWN, A_CENTER, A_TOP);
		
		setVisible(false);
	}
	
	private int getScrollerWidth() {
		return width>0?width:scrollerWidth;
	}

	@Override
	public void setVisible(boolean visible, Resource animation) {
		pageUp.setVisible(visible, animation);
		pageUp.getHighlights().setVisible(H_PAGEUP, H_VIS_FALSE);
		pageDown.setVisible(visible, animation);
		pageDown.getHighlights().setVisible(H_PAGEDOWN, H_VIS_FALSE);
		super.setVisible(visible, animation);
	}

	public void updateScroller(
			int totalRows, int topRowIndex, int bottomRowIndex, Resource scrollAnim) {
		
//		example:  10 rows, row 2 through 9 are showing (only row 0,1 are not showing)
		// lastRowShowing! (9==(10-1))
		// 8 visible rows (9-2+1)
		boolean firstRowShowing = (topRowIndex <= 0);
		boolean lastRowShowing = (bottomRowIndex >= (totalRows-1));
		int scrollHeight = -1;
		int scrollerMinHeight = 20;
		int scrollTop = 0;

		if(totalRows > 0) {
			int visibleRowCount = bottomRowIndex - topRowIndex+1;
			float percentDisplayed = ((float) visibleRowCount / (float) totalRows);
			float positionPercent = 0.0f;
			if (topRowIndex >= 0) {
				positionPercent = ((float) topRowIndex / (float) (totalRows - (bottomRowIndex - topRowIndex)));
			}
			
			//TODO tweak this so thumb is rendered with a bar of some sort in the middle of two ends
			scrollHeight = (int) (percentDisplayed * this.getHeight());
			scrollHeight = Math.max(scrollHeight, scrollerMinHeight);
			// save scrollheight so it only shrinks, never grows
			if (this.scrollMaxHeight < 0) {
				this.scrollMaxHeight = scrollHeight;
			} else {
				this.scrollMaxHeight = Math.min(this.scrollMaxHeight, scrollHeight);
				scrollHeight = this.scrollMaxHeight;
			}
			scrollTop = (int) (positionPercent * (this.getHeight() - scrollHeight));
		}
		
		if(scrollHeight < 0) {
			 scrollHeight = this.getHeight();
		}
		
		
		if(firstRowShowing && lastRowShowing) {
			setVisible(false);
		} else {
			setVisible(true);
			// note: used to do position separately so that animates but size
			// doesn't, but this should be OK and is more hme efficient
			scroller.setBounds(scroller.getX(), scrollTop, scroller.getWidth(), scrollHeight, scrollAnim);
		}
		pageUp.getHighlights().setVisible(H_PAGEUP,
				firstRowShowing ? H_VIS_FALSE : H_VIS_TRUE);
		pageDown.getHighlights().setVisible(H_PAGEDOWN,
				lastRowShowing ? H_VIS_FALSE : H_VIS_TRUE);
	}

//	public BView getScroller() {
//		return scroller;
//	}
//	
//	public BView getPageUp() {
//		return pageUp;
//	}
//	
//	public BView getPageDown() {
//		return pageDown;
//	}

}
