package com.blackledge.david.html.hme;

import java.awt.Color;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.Vector;

import com.blackledge.david.html.hme.WebBrowserApp.MEDIA;
import com.blackledge.david.html.hme.host.HMEItem;
import com.blackledge.david.tivo.gadgets.StringGetterScreen;
import com.blackledge.david.tivo.widgets.ActionButton;
import com.blackledge.david.tivo.widgets.typo.TyPo;
import com.blackledge.david.tivo.widgets.typo.TyPo.Keyboard;

import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BButton;
import com.tivo.hme.bananas.BEvent;
import com.tivo.hme.bananas.BKeyboard;
import com.tivo.hme.bananas.BList;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.HmeEvent.FontInfo;
import com.tivo.hme.sdk.IHmeProtocol;
import com.tivo.hme.sdk.StreamResource;

public class BookmarkScreen extends BScreen {

	private BList list;
	private LinkedHashMap<String, URL[]> bookmarks;
	private int bookmarkStartIndex;
	private TyPo typo;
	private String typedURL = "http://"; //$NON-NLS-1$
	private BKeyboard bkey;
	private ActionButton cancel;
	private ActionButton ok;
	private boolean isTypo = true;

	private static void debugOut(String output) {
		WebBrowserApp.debugOut(output);
	}
	
	private static int getRowFontSize(BScreen screen) {
		float ratio = ((float) screen.getHeight()) / 480f;
		return (int) (15*(1.0+(ratio-1.0)/2));
	}

	private static class BookmarkList extends BList {

		private BookmarkScreen screen;

		public BookmarkList(BookmarkScreen screen, BView parent, int x, int y,
				int width, int height, int rowHeight) {
			super(parent, x, y, width, height, rowHeight);
			this.screen = screen;
		}

		/**
		 * Handle focus movement.
		 */
		public boolean handleFocus(boolean isGained, BView gained, BView lost) {
			// work around a bug in BList when gained is null.
			if (!isGained && gained == null) {
				refresh();
				return true;
			} else {
				return super.handleFocus(isGained, gained, lost);
			}
		}
		
		@Override
		protected void createRow(BView parent, int index) {
//			debugOut("create row " + index);
			if (screen.bookmarks == null || index < size()) {
				if (index < screen.bookmarkStartIndex) {
					BButton button = new BButton(parent, 0, 0,
							parent.getWidth(), parent.getHeight());
					button.setTranslation(10, 0);
					String text = String.valueOf(get(index));
					int fontSize = this.getRowHeight();
					fontSize = getRowFontSize(getScreen())+5;
					String fontString = "default-"+(fontSize)+"-bold.font"; //$NON-NLS-1$ //$NON-NLS-2$
					restrictWidth(text, fontString, button.getWidth());
					button.setResource(button.createText(
							fontString, Color.white,
							HTMLBreakdown.fixTextForRendering(text)), RSRC_HALIGN_LEFT|RSRC_VALIGN_CENTER);
//debugOut("action:" + get(index)+" "+screen.bookmarkStartIndex);
				} else if (screen.bookmarks != null) {
//					debugOut("bkmrk " + get(index));
					BButton button = new BButton(parent, 0, 0,
							parent.getWidth(), parent.getHeight());
					Entry<String, URL[]> bookmark = screen.getBookmark(index
							- screen.bookmarkStartIndex);
					String title = bookmark.getKey();
					boolean isHME = bookmark.getValue()[0].getProtocol()
							.startsWith("hme"); //$NON-NLS-1$
					Color color = isHME ? Color.white : Color.yellow;
					int fontSize = this.getRowHeight();
					String fontString = "default-"+fontSize+".font"; //$NON-NLS-1$ //$NON-NLS-2$
					title = restrictWidth(title, fontString, button.getWidth());
					if (bookmark.getValue().length > 1
							&& bookmark.getValue()[1] != null) {
						int iconSize = 24;
						BView view = new BView(button, 10, (button.getHeight() - iconSize) / 2, iconSize,
								iconSize);
//TODO don't set resource if it's no good or something... simulator seems to think we're setting it
						view.setResource(bookmark.getValue()[1],
								HmeEvent.RSRC_IMAGE_BESTFIT
										| HmeEvent.RSRC_HALIGN_RIGHT
										| HmeEvent.RSRC_VALIGN_CENTER);
						title = "    "+title; //$NON-NLS-1$
					} else if(isHME) {
						//FIXME if icon is null, and url is hme:, try to get the icon.png
					}
					button.setResource(button.createText(fontString,
							color, HTMLBreakdown.fixTextForRendering(title)));
				}
			} 
//			else debugOut("huh?");
		}

		private String restrictWidth(String title, String fontString, int width) {
			if(width <= 10 || title == null) return title;
			FontInfo fontInfo = ((WebBrowserApp)getApp()).getFontInfos().get(fontString);
			if(fontInfo != null) {
				while(fontInfo.measureTextWidth(title)>width) {
					title = title.substring(0, title.length()-1);
				}
				return title;
			} else {
				int pxPerChar = 12;
				if(title.length() > width/pxPerChar) {
					return title.substring(0, width/pxPerChar);
				}
			}
			return title;
		}

		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			switch (code) {
			case KEY_ADVANCE:
				if(size() <= 0)
					return false;
				if(getFocus() >= size()-1) {
					setFocus(0, true);
				} else {
					setFocus(size()-1, true);
				}
				play("select.snd"); //$NON-NLS-1$
				return true;
			case KEY_INFO:
				screen.showWebPageMenu();
				return true;
			case KEY_SELECT:
			case KEY_FORWARD:
				postEvent(new BEvent.Action(this, "LINK")); //$NON-NLS-1$
				return true;
			case KEY_ENTER:
				postEvent(new BEvent.Action(this, "edit")); //$NON-NLS-1$
				return true;
			case KEY_REVERSE:
			case KEY_PLAY:
				postEvent(new BEvent.Action(this, "back")); //$NON-NLS-1$
				return true;
			case KEY_THUMBSDOWN:
			case KEY_CLEAR:
				postEvent(new BEvent.Action(this, "downgrade")); //$NON-NLS-1$
				return true;
			case KEY_THUMBSUP:
				postEvent(new BEvent.Action(this, "upgrade")); //$NON-NLS-1$
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}
	}
	public BookmarkScreen(BApplication app) {
		super(app);
		// this.getBelow().setResource(Color.yellow);
		WebBrowserApp wba = (WebBrowserApp)getApp();
		int innerWidth = this.getNormal().getWidth() - (2 * wba.getSafeTitleHorizontal());
		int innerHeight = this.getNormal().getHeight() - (2 * wba.getSafeTitleVertical());
		BView banner = new BView(this.getNormal(), 
				wba.getSafeTitleHorizontal()+wba.getSafeLogoToScrollTitle(), wba.getSafeTitleVertical(),
				innerWidth-wba.getSafeLogoToScrollTitle(), wba.getSafeTitleToScrollTop());
		BView title = new BView(banner, 0, 0, banner.getWidth(), 20);
//debugOut("notice"+WebBrowserApp.NOTICE+":"+WebBrowserApp.BROWSER_NAME);
		title.setResource(
				title.createText("system-"+(getRowFontSize(this)+1)+".font", Color.white,  //$NON-NLS-1$ //$NON-NLS-2$
						MessageFormat.format(Messages.getString("BookmarkScreen.Title"), WebBrowserApp.NOTICE, BOOKMARK_NAME) //$NON-NLS-1$
						), RSRC_HALIGN_CENTER
						| RSRC_VALIGN_BOTTOM );
		BView hints = new BView(banner, 0, title.getHeight(), banner.getWidth(),
				banner.getHeight() - title.getHeight());
		hints.setResource(
				hints.createText(
						"default-"+(getRowFontSize(this)-2)+".font", //$NON-NLS-1$ //$NON-NLS-2$
						Color.LIGHT_GRAY,
						MessageFormat.format(Messages.getString("BookmarkScreen.hints"), BOOKMARK_NAME)), //$NON-NLS-1$
				RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER | RSRC_TEXT_WRAP);

		BView listArea = new BView(this.getNormal(), wba.getSafeTitleHorizontal(), 
				wba.getSafeTitleVertical() + banner.getHeight(), innerWidth, 
				innerHeight - banner.getHeight() - wba.getSafeTitleToScrollBottom());

		int buttonHeight = getBApp().getSkin().get(H_BAR).getHeight();
		list = new BookmarkList(this, listArea, 0, 0, listArea.getWidth(),
				listArea.getHeight(), buttonHeight/2);
		list.setBarAndArrows(BAR_HANG, BAR_HANG, "back", "LINK"); //$NON-NLS-1$ //$NON-NLS-2$

		setFocusDefault(list);
		setupList();
	}

	private Entry<String, URL[]> getBookmark(int index) {
		int i = 0;
		for (Entry<String, URL[]> entry : this.bookmarks.entrySet()) {
			if (i == index)
				return entry;
			++i;
		}
		return null;
	}
	
	private void renameBookmark(int index, String title) {
		Entry<String, URL[]> entry = removeBookmark(index);
		this.bookmarks.put(title, entry.getValue());
	}
	
	private void addBookmark(URL[] url, String title) {
		this.bookmarks.put(title, url);
	}
	
	private Entry<String, URL[]> removeBookmark(int index) {
		Entry<String, URL[]> result = null;
		int i = 0;
		for (Entry<String, URL[]> entry : this.bookmarks.entrySet()) {
			if (i == index) {
				result = entry;
				break;
			}
			++i;
		}
		this.bookmarks.remove(result.getKey());
		return result;
	}
	
	private String lastVisitedTitle = ""; //$NON-NLS-1$
	private URL lastVisitedURL = null;
	public boolean isFirstPage;
	
//	/** Deal with a bug in BList focus handling. */
//	@Override
//	public void setFocus(BView focus) {
//		try {
//			super.setFocus(focus);
//		} catch(NullPointerException ex) {
//			debugOut("NPE during setFocus of "+focus);
//		}
//	}

	@Override
	public boolean handleEnter(Object arg, boolean isReturn) {
		((WebBrowserApp)getBApp()).hidePleaseWait();
		showBackground();
		
		if(isReturn) {
			if(handleReturnArg(arg))
				return true;
			// else
			// continue with re-display if this is just returning from a web
			// page (only happens when this is the very first screen)
		}
		
		this.bookmarks = ((WebBrowserApp) getBApp()).getBookmarks();
		this.lastVisitedURL = ((WebBrowserApp) getBApp()).getLastVisited();
		this.lastVisitedTitle = ((WebBrowserApp) getBApp()).getLastVisitedTitle();
		if(lastVisitedURL == null) {
			try {
				lastVisitedURL = new URL("http://www.google.com"); //$NON-NLS-1$
				lastVisitedTitle = "(sample)Google"; //$NON-NLS-1$
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		if (this.bookmarks.size() == 0) {
			addDefaultBookmarks();
		}

		setupList();

		return super.handleEnter(arg, isReturn);
	}
	
	private void showWebPageMenu() {
		BScreen menu = new WebMenuScreen(getBApp());
		// can't use TRANSITION_FORWARD because many menu commands cause a
		// screen replacement which fires during handleEnter BEFORE the pop
		// transition is processed
		getBApp().push(menu, TRANSITION_NONE);
	}
	
	private void showBackground() {
		if(((WebBrowserApp)getBApp()).isShowingLiveTV()) {
			//FIXME have a good background image instead of this color
			getBelow().setResource(new Color(80, 0, 80));
		} else {
			StreamResource videoLoop = createStream(WebBrowserApp.STREAM_LOOP_PURPLE_SHOWCASES, "video/mpeg", true); //$NON-NLS-1$
			getBelow().setResource(videoLoop);
			flush();
		}
	}
	private void disposeBackground() {
		if(((WebBrowserApp)getBApp()).isShowingLiveTV()) {
		} else {
			if(getBelow().getResource() != null) {
				getBelow().getResource().remove();
				getBelow().clearResource();
				flush();
			}
		}
	}

	@Override
	public boolean handleExit() {
		disposeBackground();
		return super.handleExit();
	}

	private void addDefaultBookmarks() {
		if(this.bookmarks == null) {
			return;
		}
		LinkedHashMap<String, URL[]> bookmarks = getDefaultBookmarks();
		
		this.bookmarks.putAll(bookmarks);
	}

	private LinkedHashMap<String, URL[]> getDefaultBookmarks() {
		LinkedHashMap<String, URL[]> bookmarks = new LinkedHashMap<String, URL[]>();
		try {
			addDefaultWebsiteBookmarks(bookmarks);
			
//			addDemoBookmarks(bookmarks);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return bookmarks;
	}

	public static void addDemoBookmarks(LinkedHashMap<String, URL[]> bookmarks)
			throws MalformedURLException {
		URL url;
		URL icon;
					url = new URL("hme://wmcbrine.com/reversi"); //$NON-NLS-1$
					icon = new URL("http://wmcbrine.com/images/reversi.png"); //$NON-NLS-1$
					bookmarks.put("launch Reversi game by wmcbrine", new URL[] {url, icon}); //$NON-NLS-1$
					
		//			url = new URL("hme://168.143.122.49:7288/mp3tunes");
					url = new URL("hme://www.mp3tunes.com"); // points to .49 ip //$NON-NLS-1$
					icon = new URL("http://168.143.122.40:7288/mp3tunes/icon.png"); //$NON-NLS-1$
					bookmarks.put("launch ('hme://...') MP3Tunes App [:( unplugged 12/2012]", new URL[] {url, icon}); //$NON-NLS-1$
		
		//			url = new URL("http://sideload.com/#primarynav");
		//			icon = new URL("http://s.mp3tunes.com/images/locker/sideload_logo.gif");
		//			bookmarks.put("(temp)sideload.com - add free MP3s to MP3Tunes", new URL[] {url, icon});
		
					// ticketchannel.tv still serves TiVoConnect, but no app at pointed location
		//			url = new URL("hme://69.72.234.59:7288/hme");
		//			url = new URL("hme://69.72.234.62:7288/hme"); // listed in ticketchannel simulator jnlp launcher
		//			url = new URL("hme://ticketchannel.tv"); //points to .59 ip
		//			bookmarks.put("(temp)TicketChannel TiVo App", new URL[] {url, icon});
		
		//			// apps.tv configures based on TiVo id, we're not a tivo.
		//			// also, the website is auctioned off as of 9/12/12
		//			url = new URL("hme://209.97.196.52:7288/submanager");
		////			url = new URL("hme://ns.apps.tv"); // or mail.apps.tv or mkloop.com (convert graphic to 1-frame mpg!)
		//			// only mkloop.com exists as of 9/3/2012, and that won't give you TiVoConnect
		//			//however hme handler now converts host to ip address before it begins.
		//			// mkloop.com gone as of 9/6/2012
		//			// no names available for apps.tv, and app is gone, now, too
		//			bookmarks.put("(temp)launch apps.tv App", new URL[] {url, icon});
		
		//			url = new URL("hme://204.176.49.116:7288/CreditCard");
		//			icon = null;
		//			bookmarks.put("(temp)launch TiVo's (hidden) Credit Card Update (PCI enabled) App", new URL[] {url, icon});
		//			url = new URL("hme://204.176.49.116:7288/TivoUpgrade");
		//			icon = null;
		//			bookmarks.put("(temp)launch TiVo's Buy a TiVo Premiere App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/amazon"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's (hidden) Amazon App", new URL[] {url, icon}); //$NON-NLS-1$
					
					url = new URL("hme://204.176.49.116:7288/bundle-netflix"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's (hidden) netflix App", new URL[] {url, icon}); //$NON-NLS-1$
					
					url = new URL("hme://204.176.49.116:7288/dominos"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's (hidden) Order Domino's Pizza Now App", new URL[] {url, icon}); //$NON-NLS-1$
		//			url = new URL("hme://204.176.49.116:7288/fandivo");
		//			icon = null;
		//			bookmarks.put("launch TiVo's Browse and Buy Movie Tickets App", new URL[] {url, icon});
		//			url = new URL("hme://204.176.49.116:7288/hdphotos");
		//			icon = null;
		//			bookmarks.put("launch TiVo's Photos & Slideshows App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/hdui"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's (hidden) TiVo Search App (hdui)", new URL[] {url, icon}); //$NON-NLS-1$
		//			url = new URL("hme://204.176.49.116:7288/hmedown");
		//			icon = null;
		//			bookmarks.put("launch TiVo's (hidden) Hme Down App", new URL[] {url, icon});
		//			url = new URL("hme://204.176.49.116:7288/kidzone");
		//			icon = null;
		//			bookmarks.put("launch TiVo's (hidden) Guru Guides App", new URL[] {url, icon});
					// same basic url for both
		//			bookmarks.put("launch TiVo's (hidden) KidZone App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/launcher"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's TiVo Search Beta App", new URL[] {url, icon}); //$NON-NLS-1$
					
					url = new URL("hme://204.176.49.116:7288/live365"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's Live365 App", new URL[] {url, icon}); //$NON-NLS-1$
					
		//			url = new URL("hme://204.176.49.116:7288/musicchoice");
		//			icon = null;
		//			bookmarks.put("launch TiVo's Music Videos from Music Choice App", new URL[] {url, icon});
		//			url = new URL("hme://204.176.49.116:7288/norhapsody");
		//			icon = null;
		//			bookmarks.put("launch TiVo's Rhapsody Music Service App placeholder (norhapsody)", new URL[] {url, icon});
					
					url = new URL("hme://204.176.49.116:7288/pandora"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's Pandora App", new URL[] {url, icon}); //$NON-NLS-1$
		//			url = new URL("hme://204.176.49.116:7288/podcast");
		//			icon = null;
		//			bookmarks.put("launch TiVo's Podcaster App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/samegame"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's SameGame App", new URL[] {url, icon}); //$NON-NLS-1$
					url = new URL("hme://204.176.49.116:7288/skullbones"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's Skull & Bones App", new URL[] {url, icon}); //$NON-NLS-1$
		//			url = new URL("hme://204.176.49.116:7288/survey");
		//			icon = null;
		//			bookmarks.put("launch TiVo's (hidden) Survey App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/swivel"); //$NON-NLS-1$
					icon = null;
					// two versions of this
					bookmarks.put("launch TiVo's (hidden) Swivel App", new URL[] {url, icon}); //$NON-NLS-1$
					url = new URL("hme://204.176.49.116:7288/tivocast"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's (hidden) TiVoCast App (video on demand)", new URL[] {url, icon}); //$NON-NLS-1$
		//			url = new URL("hme://204.176.49.116:7288/videoplayer");
		//			icon = null;
		//			bookmarks.put("launch TiVo's (hidden) Video Player App", new URL[] {url, icon});
					url = new URL("hme://204.176.49.116:7288/wordsmith"); //$NON-NLS-1$
					icon = null;
					bookmarks.put("launch TiVo's WordSmith App", new URL[] {url, icon}); //$NON-NLS-1$
					url = new URL("hme://204.176.49.116:7288/youtube"); //$NON-NLS-1$
					icon = null;
					// two versions, second one labeled "tcast jump"
					bookmarks.put("launch TiVo's (hidden) YouTube App", new URL[] {url, icon}); //$NON-NLS-1$
					
//					url = new URL("hme://204.176.49.116/flash/youtube_leanback");
//					icon = null;
//					bookmarks.put("(temp)launch FLASH version of TiVo's (hidden) YouTube App", new URL[] {url, icon});
					
		/*
		urls: <url>
		 https://204.176.49.59:7287/CreditCard_20080912-20080912_131340/1221250421/1289849840-1605209840/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.59:7287/TivoUpgrade-20120424_122953/1335295796/1335399021-1650759021/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/amazon-1-9.2011.03.16-0632-20110316_154048/1300315250/1317939394-1633299394/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/dominos.2009.05.07.r1021.logerror.signed-20090507_174812/1241743696/1289849802-1605209802/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/fandivo-1.1.2011.11.10-1645-20111115_112508/1321385109/1343238702-1658598702/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/hdphotos-ml_2010.09.13-1339-20100913_134517/1284410719/1343239081-1658599081/clientip_67.0.59.230/
		</url>
		<url>
		 http://204.176.49.116:7288/hmedown_20080317-20080402_170823/1207181304/1240249987-1587405187/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/kidzone_v1.0-20100622-20100622_141902/1277241543/1289849391-1605209391/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/kidzone_v1.0-20100622-20100622_141902/1277241543/1289849349-1605209349/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/live365-1.0_2010.08.02-2340-20100802_234348/1280817829/1343239177-1658599177/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/musicchoice-1.0-2010.08.04-0632-20100804_161353/1280963634/1328046275-1643406275/clientip_67.0.59.230/
		</url>
		<url>
		 http://204.176.49.116:7288/podcast-20100730-qbert-20100730_180250/1280538174/1343239384-1658599384/clientip_67.0.59.230/
		</url>
		<url>
		 http://204.176.49.116:7288/samegame_20071012_01-20080402_173700/1207183021/1343239500-1658599500/clientip_67.0.59.230/
		</url>
		<url>
		 http://204.176.49.116:7288/skullbones_20071012-20080402_173743/1207183066/1343239554-1658599554/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/survey_20080530-01_signed-20080530_150815/1212185297/1289848835-1605208835/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-20120508_094009/1336495211/1336595307-1651955307/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/swivel-2.8.3-2012-05-08-0603at553106-20120508_094009/1336495211/1336595232-1651955232/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/tivocast-2.8-2012-06-26-0603at563743-20120626_120132/1340737293/1344881428-1660241428/clientip_67.0.59.230/
		</url>
		<url>
		 https://204.176.49.65:7287/videoplayer-0023-release-sst-20090716_122445/1247772287/1333658049-1649018049/clientip_67.0.59.230/
		</url>
		<url>
		 http://204.176.49.116:7288/wordsmith_20071012-20080402_180020/1207184421/1343239627-1658599627/clientip_67.0.59.230/
		</url>			
		 */
	}

	public static void addDefaultWebsiteBookmarks(LinkedHashMap<String, URL[]> bookmarks)
			throws MalformedURLException {
		URL url;
		URL icon;
		url = new URL("https://docs.google.com/document/pub?id=1o0jVDy2Ek-baZ7sm3v9OtTxZ9MksS-7RzLCY_1GU51w"); //$NON-NLS-1$
		icon = null;
		bookmarks.put("About "+WebBrowserApp.BROWSER_NAME, new URL[]{url, icon}); //$NON-NLS-1$

		if(WebBrowserApp.getDebugLevel() > 0) {
			url = new URL("http://192.168.0.20:80/EnterWebzTest");
			icon = null;
			bookmarks.put("(temp)Test HTML Page", new URL[] {url, icon});
		}
		/*
 "type":"uiNavigate",
 "uiDestinationType":"hme",
 "uri":"x-tivo:hme:uuid:863cb78f-efdd-4106-b572-51733983dc76",
 "parameters":{
  "title":"Title goes here",
  "subtitle":"Subtitle goes here",
  "description":"Description goes here",
  "uri":"http://192.168.10.198:9032/videos/movies/star_wars2.mpg?Format=video%2Fx-tivo-mpeg",
  "duration":8280
 }
		 * */

//			url = new URL("hme://192.168.0.20:7288/webz/?url=http://voice.google.com&test=blah");
//			icon = null;
//			bookmarks.put("(temp)Test HME Transition", new URL[] {url, icon});
		
		// TODO look up host name to get IP, THEN make request for IP/TiVoConnect... that works in the old apps.tv case

		url = new URL("http://www.google.com"); //$NON-NLS-1$
		icon = new URL("http://www.google.com/images/google_favicon_128.png"); //$NON-NLS-1$
		bookmarks
				.put("Google", //$NON-NLS-1$
						new URL[] {url, icon});
		
		url = new URL("http://hmedev.wikidot.com/enter-webz-links#main-content"); //$NON-NLS-1$
		icon = null;
		bookmarks.put("Enter Webz suggested links", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://www.youtube.com"); //$NON-NLS-1$
		icon = new URL("http://s.ytimg.com/yts/img/favicon_32-vflWoMFGx.png"); //$NON-NLS-1$
		bookmarks.put("YouTube", new URL[] {url, icon}); //$NON-NLS-1$
		
//			
//url = new URL("http://feeds.feedburner.com/ICanHasCheezburger");
//icon = null;
//bookmarks.put("rss cheezburger",  new URL[]{url, icon});
//
//url = new URL("http://feeds.feedburner.com/AnimalGifs");
//icon = null;
//bookmarks.put("rss animal gifs",  new URL[]{url, icon});

		url = new URL("http://en.m.wikipedia.org"); //$NON-NLS-1$
		icon = new URL("http://en.wikipedia.org/apple-touch-icon.png"); //$NON-NLS-1$
		bookmarks.put("Wikipedia (mobile)", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://m.wund.com/"); //$NON-NLS-1$
		icon = new URL("http://icons.wunderground.com/graphics/smalllogo.gif"); //$NON-NLS-1$
		bookmarks.put("(enter zip then re-THUMB)Weather Underground (mobile)", new URL[] {url, icon}); //$NON-NLS-1$
		
//			url = new URL("https://www.google.com/voice/m/i/voicemail");
		url = new URL("https://www.google.com/voice/m"); //$NON-NLS-1$
		icon = null;//new URL();
		bookmarks.put("Google Voice", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://mail.google.com/mail/x/"); //$NON-NLS-1$
		icon = new URL("http://mail.google.com/mail/images/2/mail_icon_32.png"); //$NON-NLS-1$
		bookmarks.put("Gmail (mobile)", new URL[] {url, icon}); //$NON-NLS-1$
		
		url=new URL("http://mail.google.com/mail/x/?ui=html"); //$NON-NLS-1$
		icon = new URL("http://mail.google.com/mail/images/2/mail_icon_32.png"); //$NON-NLS-1$
		bookmarks.put("Gmail full basic HTML", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://m.facebook.com/"); //$NON-NLS-1$
		icon = null;
		bookmarks.put("Facebook (mobile)", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://imdb.com"); //$NON-NLS-1$
		icon = new URL("http://i.media-imdb.com/images/SF0ae8798307d147d8ea59044184b4339e/mobile/icon_114x114.png"); //$NON-NLS-1$
		bookmarks.put("Internet Movie Database (IMDB)", new URL[] {url, icon});			 //$NON-NLS-1$

		url = new URL("http://archive.org/details/feature_films"); //$NON-NLS-1$
		icon = new URL("http://archive.org/images/glogo.jpg"); //$NON-NLS-1$
		bookmarks.put("Feature Films at Archive.org", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://archive.org/details/etree"); //$NON-NLS-1$
		icon = new URL("http://archive.org/images/glogo.jpg"); //$NON-NLS-1$
		bookmarks.put("Live Concerts at Archive.org", new URL[] {url, icon}); //$NON-NLS-1$

		url = new URL("http://archive.org/details/classic_tv"); //$NON-NLS-1$
		icon = new URL("http://archive.org/images/glogo.jpg"); //$NON-NLS-1$
		bookmarks.put("Classic TV at Archive.org", new URL[] {url, icon}); //$NON-NLS-1$

		url = new URL("http://archive.org/details/americana"); //$NON-NLS-1$
		icon = new URL("http://archive.org/images/glogo.jpg"); //$NON-NLS-1$
		bookmarks.put("American Library texts at Archive.org", new URL[] {url, icon}); //$NON-NLS-1$

		url = new URL("http://tivocommunity.com"); //$NON-NLS-1$
		icon = new URL("http://www.tivocommunity.com/home/sites/default/files/tcflogonew5.png"); //$NON-NLS-1$
		bookmarks.put("TiVoCommunity.com", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("https://mobile.twitter.com/TiVo"); //$NON-NLS-1$
		icon = new URL("https://ma.twimg.com/twitter-mobile/e85c3330835e527a4e9f0f9621fc53841a65c1b8/images/ic_favicon.png"); //$NON-NLS-1$
		bookmarks.put("Twitter: TiVo", new URL[] {url, icon}); //$NON-NLS-1$
		
		url = new URL("http://xkcd.com"); //$NON-NLS-1$
		icon = new URL("http://imgs.xkcd.com/static/apple-touch-icon-precomposed.png"); //$NON-NLS-1$
		bookmarks.put("xkcd comics", new URL[] {url, icon}); //$NON-NLS-1$
		
//			url = new URL("http://icanhascheezburger.com/#MobileSearch");
//			icon = null;
//			bookmarks.put("BROKEN LOLcats - i can has cheezburger?", new URL[] {url, icon});
		
		url = new URL("http://david.blackledge.com"); //$NON-NLS-1$
		icon = null;
		bookmarks.put("David.Blackledge.com", new URL[] {url, icon}); //$NON-NLS-1$
		
		addDemoBookmarks(bookmarks);
	}
	
	private static final String ENTER_URL = Messages.getString("BookmarkScreen.Enter_URL"); //$NON-NLS-1$
	public static final String BOOKMARK_NAME = Messages.getString("BookmarkScreen.BOOKMARK_NAME"); //$NON-NLS-1$

	private void setupList() {
		int oldFocus = list.getFocus();
		if(oldFocus < 0) oldFocus = 0;
		list.clear();
		// add other entries and move bookmarkstartindex
		if(lastVisitedURL != null) {
			list.add(MessageFormat.format(Messages.getString("BookmarkScreen.Last_Visited"),lastVisitedTitle)); //$NON-NLS-1$
		}
		list.add(ENTER_URL);
		this.bookmarkStartIndex = 2;//list.size();

		if(bookmarks == null || bookmarks.size() == 0) {
			addDefaultBookmarks();
		}
		if (bookmarks != null) {
			list.add(new Vector<String>(bookmarks.keySet()));
		}
		if(oldFocus >= list.size()) {
			oldFocus = list.size()-1;
		}
		list.setFocus(oldFocus, false);
	}

	@Override
	public boolean handleAction(BView view, Object action) {
		if (action.equals("LINK")) { //$NON-NLS-1$
			int index = list.getFocus();
			if (index < this.bookmarkStartIndex) {
				if(list.get(index).equals(ENTER_URL)) {
					setUpKeyboard();
					
					if(isTypo) {
						typo.setKeyboard(Keyboard.URL);
						typo.setActions("newURL", "urlCancel"); //$NON-NLS-1$ //$NON-NLS-2$
						typo.setValue(this.typedURL);
					} else {
						bkey.setValue(this.typedURL);
						cancel = new ActionButton(getAbove(), ((WebBrowserApp)getApp()).getSafeTitleHorizontal(), 150, 200, 20, "urlCancel"); //$NON-NLS-1$
						cancel.setBarAndArrows("urlCancel", "right", null, "down", false); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						new BView(cancel, 5, 5, cancel.getWidth()-10, cancel.getHeight()-10).setResource(cancel.createText("default-"+getRowFontSize(this)+".font", Color.white, "Cancel")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						ok = new ActionButton(getAbove(), ((WebBrowserApp)getApp()).getSafeTitleHorizontal()+250, 150, 200, 20, "newURL"); //$NON-NLS-1$
						ok.setBarAndArrows("left",  "newURL", null, "down", false); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						new BView(ok, 5, 5, ok.getWidth()-10, ok.getHeight()-10).setResource(ok.createText("default-"+getRowFontSize(this)+".font", Color.white, "Ok")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
				} else {
					// last visited.
					
					replaceBookmarkScreen(lastVisitedURL);
//					// replace the bookmark screen with the web screen
//					// if we're the first screen, pop does nothing.
////					getBApp().pop();
////					getBApp().push(
//					if(isFirstPage) {
//						getBApp().push(
//								new WebScreen(getBApp()), TRANSITION_FORWARD,
//								lastVisitedURL);
//					} else {
//					getBApp().replaceCurrentScreen(
//							new WebScreen(getBApp()), TRANSITION_FORWARD,
//							lastVisitedURL);
//					}
					
				}
			} else {
				index -= this.bookmarkStartIndex;
				Entry<String, URL[]> bookmark = getBookmark(index);
				if(bookmark.getValue()[0].getProtocol().equals("hme")) { //$NON-NLS-1$
					((WebBrowserApp)getBApp()).loadApp(bookmark.getValue()[0]);
					return true;
				}
				replaceBookmarkScreen(bookmark.getValue());
			}
			return true;
		} else if (action.equals("back")) { //$NON-NLS-1$
			if(isFirstPage) {
				((WebBrowserApp)getBApp()).closeBrowser();
			} else {
				if(getBApp().getCurrentScreen() == null) debugOut("current screen is null! BEFORE pop!"); //$NON-NLS-1$
				getBApp().pop();
				if(getBApp().getCurrentScreen() == null) debugOut("current screen is null! after pop"); //$NON-NLS-1$
			}
			return true;
		} else if (action.equals("edit")) { //$NON-NLS-1$
			int index = list.getFocus();
			if (index < this.bookmarkStartIndex) {
			} else {
				index -= this.bookmarkStartIndex;
				Entry<String, URL[]> bookmark = getBookmark(index);
				
				setUpKeyboard();
				if(isTypo ) {
//					//					this.getAbove().setResource(Color.darkGray);
//					this.getNormal().setTransparency(0.5f);
//					typo = new TyPo(this.getAbove(), getApp().getSafeTitleHorizontal(), 200, this
//							.getAbove().getWidth() - (2 * getApp().getSafeTitleHorizontal())-75, 15,
//							true);
//					typo.setExitOptions(TyPo.EXIT_CANCEL, TyPo.EXIT_FINISHED,
//							true, true, false, false);
//					typo.setActivateOnFocus(true);
//					this.setFocus(typo);
//					typo.setFocus();
//					// prevent NPE when typo's list clears and set focus to the default
//					setFocusDefault(typo);
					
					typo.setKeyboard(Keyboard.DEFAULT);
					typo.setActions("editTitle", "editCancel"); //$NON-NLS-1$ //$NON-NLS-2$
					typo.setValue(bookmark.getKey());
				}//TODO add non-typo if needed.
			}
			return true;
		} else if (action.equals("downgrade")) { //$NON-NLS-1$
			int index = list.getFocus();
			if (index < this.bookmarkStartIndex) {
			} else {
				//TODO undo further upgrades when added
				index -= this.bookmarkStartIndex;
				// remove the bookmark from the list
				Entry<String, URL[]> bookmark = removeBookmark(index);
				if (bookmark != null) {
					// remove the bookmark from storage
					((WebBrowserApp) getBApp()).removeBookmark(
							bookmark.getValue()[0], bookmark.getKey(), bookmark.getValue()[1]);
					// update the list view
					// this clears the list and rebuilds it, attempting to maintain focus index.
					setupList();
					return true;
				}
			}
			return false;

		} else if (action.equals("upgrade")) { //$NON-NLS-1$
			int index = list.getFocus();
			if (index < this.bookmarkStartIndex) {
				if(index == 0) {
					String title = this.lastVisitedTitle;
					if(title.indexOf(':')>=0) {
						title = title.substring(title.indexOf(':')+1).trim();
					}
					((WebBrowserApp) getBApp()).saveBookmark(this.lastVisitedURL, title, null); // FIXME icons
					addBookmark(new URL[]{this.lastVisitedURL, null}, title);
					// update the list view
					// this clears the list and rebuilds it, attempting to maintain focus index.
					setupList();
					list.setFocus(list.size()-1, true);
					return true;
				} else {
					//TODO allow arbitrary url entry as bookmark
					return false;
				}
			} else {
				index -= this.bookmarkStartIndex;
				// remove the bookmark from the list
				Entry<String, URL[]> bookmark = getBookmark(index);
				if (bookmark != null) {
					String title = bookmark.getKey();
					if (title.startsWith("(temp")) { //$NON-NLS-1$
						int end = title.indexOf(')');
						if (end >= 0) {
							title = title.substring(end + 1);
						} else {
							//TODO add additional upgrade
							return false;
						}
						renameBookmark(index, title);
					} else {
						//TODO add additional upgrade
						return false;
					}
					// save the bookmark in storage
					((WebBrowserApp) getBApp()).saveBookmark(
							bookmark.getValue()[0], title, bookmark.getValue()[1]);
					// update the list view
					// this clears the list and rebuilds it, attempting to maintain focus index.
					setupList();
					list.setFocus(list.size()-1, true);
					return true;
				}
			}
			return false;

		}
		return super.handleAction(view, action);
	}

	private boolean handleReturnArg(Object arg) {
		if(arg == null) return false;
		String action = arg.toString();
		WebBrowserApp web = (WebBrowserApp) getBApp();
		if(action.startsWith(WebMenuScreen.MENU_PREFIX)) {
			web.handleMenuAction(action);
////			} else if(action.equals("menu_brightness")) {
////				if(shade.getTransparency() == 1.0f) {
////					shade.setTransparency(0.75f);
////				} else {
////					shade.setTransparency(1.0f);
////				}
//			} else 
				if(action.equals(WebMenuScreen.MENU_PIP)) {
				disposeBackground();
//				web.setShowLiveTV(!web.isShowingLiveTV());
				showBackground();
////			} else if(action.equals("menu_link_thumbtack")) {
////				bookmarkFocusedLink();
			}
		}
		else
	if (action.equals("newURL")) { //$NON-NLS-1$
		if(isTypo) {
			if(typo != null)
				this.typedURL = typo.getValue();
		} else {
		this.typedURL = bkey.getValue();
		}
		URL url = null;
		try {
			url = new URL(typedURL);
		} catch (IllegalArgumentException e) {
			getBApp().play("bonk.snd"); //$NON-NLS-1$
		} catch (MalformedURLException e) {
			getBApp().play("bonk.snd"); //$NON-NLS-1$
		}

		if (url != null) {
			replaceBookmarkScreen(url);
//			// replace the bookmark screen with the web screen
//			// if we're the first screen, pop does nothing.
////			getBApp().pop();
////			getBApp().push(
//			if(isFirstPage) {
//				getBApp().push(
//						new WebScreen(getBApp()), TRANSITION_FORWARD,
//						url);
//			} else {
//			getBApp().replaceCurrentScreen(
//					new WebScreen(getBApp()), TRANSITION_FORWARD,
//					url);
//			}
		}
		shutDownKeyboard();
	} else if (action.equals("editTitle")) { //$NON-NLS-1$
		int index = list.getFocus();
		if (index < this.bookmarkStartIndex) {
		} else {
			index -= this.bookmarkStartIndex;
			String newTitle = null;
			if(isTypo) {
				if(typo != null)
					newTitle = typo.getValue();
//				} else {
//				newTitle = bkey.getValue();
			}
			Entry<String, URL[]> bookmark = getBookmark(index);
			if (newTitle != null && newTitle.trim().length() > 0) {
				//FIXME create .renameBookmark instead so it keeps its spot.
				((WebBrowserApp) getBApp()).removeBookmark(
						bookmark.getValue()[0], bookmark.getKey(), bookmark.getValue()[1]);
				((WebBrowserApp) getBApp()).saveBookmark(
						bookmark.getValue()[0], newTitle, bookmark.getValue()[1]);
				
				renameBookmark(index, newTitle);
				setupList();
			}//TODO return false (after cleaning up) if we had no title.
		}
		
		shutDownKeyboard();
//		this.setFocus(list);
//		if(isTypo) {
//			// change default back
//			setFocusDefault(list);
////			this.getAbove().setResource(null);
//			this.getNormal().setTransparency(0.0f);
//			typo.remove();
//			typo = null;
////		} else {
////			this.getNormal().setVisible(true);
////			bkey.remove();
////			cancel.remove();
////			ok.remove();
////			bkey = null;
//		}

	} else if (action.equals("urlCancel") || action.equals("editCancel")) { //$NON-NLS-1$ //$NON-NLS-2$
		this.setFocus(list);
		if (isTypo) {
			// this.getAbove().setResource(null);
			this.getNormal().setTransparency(0.0f);
			((WebBrowserApp)getBApp()).closeTyPo();
//			typo.setVisible(false);
//			 typo.remove(); // now it's a app-wide resource
			 typo = null;
		} else {
			this.getNormal().setVisible(true);
			bkey.remove();
			cancel.remove();
			ok.remove();
			bkey = null;
		}
	}
		return true;
	}
	
	private void replaceBookmarkScreen(Object urls) {
		// replace the bookmark screen with the web screen
		((WebBrowserApp)getBApp()).showPleaseWait();
		// if we're the first screen, pop does nothing.
//				getBApp().pop();
//				getBApp().push(
		WebScreen webScreen = WebScreen.newWebScreen(getBApp());
		boolean pageSuccess = webScreen.prepare(urls);
		if(!pageSuccess) {
			((WebBrowserApp)getBApp()).hidePleaseWait();
			((WebBrowserApp)getBApp()).play("bonk.snd"); //$NON-NLS-1$
			webScreen.remove();
			return;
		}
		if(isFirstPage) {
			getBApp().push(
					webScreen, TRANSITION_FORWARD,
					urls);
		} else {
			getBApp().pop();
			getBApp().push(
//		getBApp().replaceCurrentScreen(
				webScreen, TRANSITION_FORWARD,
				urls);
		}
	}

	private void setUpKeyboard() {
		if(isTypo ) {
			//					this.getAbove().setResource(Color.darkGray);
			this.getNormal().setTransparency(0.5f);
			if(typo == null) {
				typo = ((WebBrowserApp)getBApp()).showTyPo();
//				TyPo.setUserWordListAllowed(false);
//				typo = new TyPo(this.getAbove(), ((WebBrowserApp)getApp()).getSafeTitleHorizontal(), 200, this
//						.getAbove().getWidth() - (2 * ((WebBrowserApp)getApp()).getSafeTitleHorizontal())-75, 15,
//						true);
			}
//			typo.setExitOptions(TyPo.EXIT_CANCEL, TyPo.EXIT_FINISHED,
//					true, true, false, false);
			
//			typo.setActivateOnFocus(true);
//			this.setFocus(typo);
//			typo.setFocus();
//			// prevent NPE when typo's list clears and set focus to the default
//			setFocusDefault(typo);

			//					getBApp().push(new StringGetterScreen(null, getBApp()),
			//							TRANSITION_FADE,
			//							new String[] { "Enter URL", this.typedURL });

		} else {
			this.getNormal().setVisible(false, getResource("*500")); //$NON-NLS-1$
			bkey = new BKeyboard(getAbove(), ((WebBrowserApp)getApp()).getSafeTitleHorizontal(), 200, this.getAbove().getWidth() - (2*((WebBrowserApp)getApp()).getSafeTitleHorizontal()), this.getAbove().getHeight()-200-((WebBrowserApp)getApp()).getSafeTitleVertical(), BKeyboard.PLAIN_KEYBOARD, false);
			this.setFocus(bkey);
			bkey.setFocus();
		}
	}

	private void shutDownKeyboard() {
		this.setFocus(list);
		if(isTypo) {
			// change default back
			setFocusDefault(list);
//				this.getAbove().setResource(null);
			this.getNormal().setTransparency(0.0f);
			((WebBrowserApp)getBApp()).closeTyPo();
//			typo.remove();
			typo = null;
		} else {
			this.getNormal().setVisible(true);
			bkey.remove();
			cancel.remove();
			ok.remove();
			bkey = null;
		}
	}

}
