package com.blackledge.david.html.hme;

import java.awt.Dimension;
import java.net.URL;

import com.blackledge.david.html.HTMLRow;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.HmeEvent.ResourceInfo;
import com.tivo.hme.sdk.IHmeEventHandler;
import com.tivo.hme.sdk.Resource;

public class InlineBackgroundResourceHandler implements IHmeEventHandler {

	private BView fgBView;
	private Resource bgresource;
	private int bgrepeat;
	private int bgposition;
	private URL bgu;

	public InlineBackgroundResourceHandler(BView fgBView, URL bgu, Resource bgresource,
			int bgrepeat, int bgposition) {
				this.fgBView = fgBView;
				this.bgu = bgu;
				this.bgresource = bgresource;
				this.bgrepeat = bgrepeat;
				this.bgposition = bgposition;
	}

	public void postEvent(HmeEvent event) {
		// watch for image size (or error), store result in WBA's cache, and call...
		if(event.getOpCode() == HmeEvent.EVT_RSRC_INFO) {
			HmeEvent.ResourceInfo riEvent = (ResourceInfo) event;
			if(riEvent.getMap() != null) {
				Object wid = riEvent.getMap().get("width");
				Object hei = riEvent.getMap().get("height");
				if(wid != null && hei != null) {
					int width = Integer.parseInt(wid.toString());
					int height = Integer.parseInt(hei.toString());
					Dimension d = new Dimension(width, height);
					
					((WebBrowserApp)fgBView.getBApp()).putResourceCache(bgu.toString(), d);
					
					adjustInlineBackgroundResource(fgBView, bgresource, bgrepeat, bgposition, width, height);
				}
			}
		}

	}

	public static void adjustInlineBackgroundResource(BView fgBView,
			Resource bgresource, int bgrepeat, int bgposition, int bgwidth, int bgheight) {
		// Now that we know the size of the image resource, update fgBView's
		// display of the resource based on repeat/position.
		
		// For repeat, just scale fgBView and adjust its size smaller so the
		// result is the same, but the image is now stretched to fit.
		if(bgrepeat == HTMLRow.ROW_BG_REPEAT_BOTH) {
			fgBView.setResource(bgresource, Resource.RSRC_HALIGN_LEFT | Resource.RSRC_VALIGN_TOP);
			int wid = fgBView.getWidth();
			int hei = fgBView.getHeight();
			float xScale = (float)wid/(float)bgwidth;
			float yScale = (float)hei/(float)bgheight;
			// repeating scale shouldn't REDUCE the size of the image, though.
			// FIXME buuuuut... we need to maintain the size of the view!
			xScale = Math.max(1.0f, xScale);
			yScale = Math.max(1.0f, yScale);
			int scaledDownWid = bgwidth;
			int scaledDownHei = bgheight;
WebBrowserApp.debugOut("bg repeat "+wid+"x"+hei+" -> "+scaledDownWid+"*"+xScale+" x "+scaledDownHei+"*"+yScale);
			fgBView.setSize(scaledDownWid, scaledDownHei);
			fgBView.setScale(xScale, yScale);
		} else if(bgrepeat == HTMLRow.ROW_BG_REPEAT_X) {
			fgBView.setResource(bgresource, Resource.RSRC_HALIGN_LEFT | Resource.RSRC_VALIGN_TOP);
			int wid = fgBView.getWidth();
			int hei = fgBView.getHeight();
			float xScale = (float)wid/(float)bgwidth;
			float yScale = 1.0f;
			// repeating scale shouldn't REDUCE the size of the image, though.
			// FIXME buuuuut... we need to maintain the size of the view! (so should do true scale for size, but max'd scale for scaling
			xScale = Math.max(1.0f, xScale);
			yScale = Math.max(1.0f, yScale);
			int scaledDownWid = bgwidth;
			int scaledDownHei = hei;
WebBrowserApp.debugOut("bg repeat-x "+wid+"x"+hei+" -> "+scaledDownWid+"*"+xScale+" x "+scaledDownHei+"*"+yScale);
			fgBView.setSize(scaledDownWid, scaledDownHei);
			fgBView.setScale(xScale, yScale);
		} else if(bgrepeat == HTMLRow.ROW_BG_REPEAT_Y) {
			fgBView.setResource(bgresource, Resource.RSRC_HALIGN_LEFT | Resource.RSRC_VALIGN_TOP);
			int wid = fgBView.getWidth();
			int hei = fgBView.getHeight();
			float xScale = 1.0f;
			float yScale = (float)hei/(float)bgheight;
			// repeating scale shouldn't REDUCE the size of the image, though.
			// FIXME buuuuut... we need to maintain the size of the view!
			xScale = Math.max(1.0f, xScale);
			yScale = Math.max(1.0f, yScale);
			int scaledDownWid = wid;
			int scaledDownHei = bgheight;
WebBrowserApp.debugOut("bg repeat-y "+wid+"x"+hei+" -> "+scaledDownWid+"*"+xScale+" x "+scaledDownHei+"*"+yScale);
			fgBView.setSize(scaledDownWid, scaledDownHei);
			fgBView.setScale(xScale, yScale);
		} else {
			// no repeat
			// nothing to do ... was already set up correctly
		}
	}

}
