package com.blackledge.david.html.hme;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;

import com.tivo.hme.interfaces.IApplication;
import com.tivo.hme.interfaces.IArgumentList;
import com.tivo.hme.interfaces.IContext;
import com.tivo.hme.interfaces.IHmeConstants;
import com.tivo.hme.interfaces.IHttpRequest;
import com.tivo.hme.interfaces.ILogger;
import com.tivo.hme.sdk.Application;
import com.tivo.hme.sdk.Factory;
import com.tivo.hme.sdk.IHmeProtocol;
import com.tivo.hme.sdk.io.FastInputStream;
import com.tivo.hme.sdk.io.HmeInputStream;
import com.tivo.hme.sdk.io.HmeOutputStream;

public class WebBrowserAppFactory extends Factory {
	private String startTitle;
	private String startURL;
	private int count;
	private static int urlCount = 0;

	/**
	 * Override to report a different HME version, and to set startURL if server
	 * instantiated with a startURL argument.
	 */
	@Override
	public IApplication createApplication(IContext context)
			throws IOException {
        int VERSION_0_49			= (0 << 8) | 49;
        int VERSION_0_40			= (0 << 8) | 40;
        int sentVersion = 
//                IHmeProtocol.VERSION;
        		VERSION_0_49;
//        		VERSION_0_40;
		IApplication application = supercreateApplication(context, sentVersion);
//			if(context instanceof HostContext) {
//				HostContext hcontext = (HostContext) context;
////				hcontext.
//			}
		WebBrowserApp app = (WebBrowserApp) application;
		//TODO currently this only acts as default if no URL arg, should this override instead?
		if(app.startURL == null) {
			app.startURL = this.startURL;
		}

		return application;
	}
	/**
	 * Copy of super factory implementation with parameterized version number to send to TiVo.
	 */
    public IApplication supercreateApplication(IContext context, int sentVersion) throws IOException
    {
	int version = -1;
        HmeOutputStream out = null;
        HmeInputStream in = null;

        synchronized (lock) {                   // lock this factory
            if (context.getOutputStream() instanceof HmeOutputStream) {
                out = (HmeOutputStream)context.getOutputStream();
	    }
            else {
                out = new HmeOutputStream(context.getOutputStream());
	    }
            
            if (context.getInputStream() instanceof HmeInputStream) {
                in = (HmeInputStream)context.getInputStream();
            } else {
                in = new HmeInputStream(context.getInputStream());
	    }
            
            // HME protocol starts here
	    // Synchronize and flush the writes, so that no other writer 
	    // interleaves within our data
	    synchronized (out.getMonitor()) {
                out.writeInt(IHmeProtocol.MAGIC);
                out.writeInt(
//                		IHmeProtocol.VERSION
                		sentVersion
                		);
                out.flush();
	    }
		
            // read magic and version
            int magic = in.readInt();
            if (magic != IHmeProtocol.MAGIC) {
                throw new IOException(
		    "bad magic: 0x" + Integer.toHexString(magic)
//		    +getIdForException());
		    );
            }
            version = in.readInt();
            if (version >> 8 < 
//            IHmeProtocol.VERSION
            sentVersion
            >> 8 ) {
                throw new IOException(
                        "version mismatch: " + 
                        ( version >> 8 ) + "." + ( version & 0xff ) +
                        " < " +
                        ( 
//                        		IHmeProtocol.VERSION
                        		sentVersion
                        		>> 8 ) + "." + ( 
//                        				IHmeProtocol.VERSION
                        				sentVersion
                        				& 0xff )
//			+ getIdForException());
                        );
            }
        } // synchronized

        // maintain proper order of locks -- we have released 
        // the factory monitor before entering the app monitor
    
        IApplication retApp = null;
        try {
//            Application app = (Application)clazz.newInstance();
        	WebBrowserApp app = (WebBrowserApp)clazz.newInstance();
            synchronized (app) {
                app.setFactory(this);
                app.setContext(context, version);
		app.setMonitor(app);
                retApp = app;
            }
        } catch (InstantiationException ex) {
	    log(ILogger.LOG_NOTICE, ex);
        } catch (IllegalAccessException ex) {
	    log(ILogger.LOG_NOTICE, ex);
        }

	out.flush();

        return retApp;
    } // createApplication()
	
	@Override
	public InputStream getStream(String uri) throws IOException {
        synchronized (lock) {
        	if (WebProxy.isStreamProxyUri( uri )) {
        		WebProxy proxy = getProxy(uri);
        		String urip = proxy.getProxyUri();
        		URL url = new URL(getAssetURI(), urip);
        		URLConnection conn = url.openConnection();
        		// set headers and referer on conn
        		if(proxy.headers != null) {
        		}
        		if(proxy.referer != null) {
        			conn.addRequestProperty("Referer", proxy.referer.toString());
        		}
        		URLStream urlStream = 
        				new URLStream(
        						conn.getInputStream(), conn.getContentLength());
        		return urlStream;
            }
        	return super.getStream(uri);
        }
	}
	//FIXME set up for proxying images for apps
	
	public WebProxy createProxy(URL original, URL referer, Map<String, List<String>> headers, String contentType, boolean forceProxy) {
		WebProxy proxy = new WebProxy(original, referer, headers, contentType, forceProxy);
		//FIXME save proxy to a list, and return existing proxy for duplicate requests
		return proxy;
	}
	
	private WebProxy getProxy(String streamUri) {
		//FIXME search through list for matching Uri and return it or null
		return null;
	}
	
	// local copy of sdk's package-private class
    /**
     * A simple stream that responds correctly to in.available(). This is for
     * http streams.
     */
    static class URLStream extends FastInputStream
    {
        long contentLength;
        URLStream(InputStream in, long contentLength)
        {
            super(in, IHmeConstants.TCP_BUFFER_SIZE);
            this.contentLength = contentLength;
        }
        public int available()
        {
            return (int) contentLength;
        }

        /**
         * This is a public entry point and must be synchronized.
         *
         * @see com.tivo.hme.interfaces.IHmeThreadSafe
         */
        public int read() throws IOException
        {
            synchronized (lock) {       // inherited from superclass
                contentLength -= 1;
                return in.read();
            }
        }

        /**
         * This is a public entry point and must be synchronized.
         *
         * @see com.tivo.hme.interfaces.IHmeThreadSafe
         */
        public int read(byte b[], int off, int length) throws IOException
        {
            synchronized (lock) {
                int n = super.read(b, off, length);
                if (n > 0) {
                    contentLength -= n;
                }
                return n;
            }
        }

        /**
         * This is a public entry point and must be synchronized.
         *
         * @see com.tivo.hme.interfaces.IHmeThreadSafe
         */
        public long skip(long n) throws IOException
        {
            synchronized (lock) {
                n = super.skip(n);
                if (n > 0) {
                    contentLength -= n;
                }
                return n;
            }
        }
    } // class URLStream
	
	
	
	@Override
	protected void init(IArgumentList args) {
		synchronized (WebBrowserAppFactory.class) {
			count = urlCount;
			++urlCount;
		}
		
		if(args.getRemainingCount() > 0) {
			startURL = args.shift();
		}
		if(args.getRemainingCount() > 0) {
			// serve app with (argument) title in this case.
			startTitle = args.shift();
		}
//			super.init(args);
	}
	
	@Override
	public String getAppTitle() {
		if(this.startTitle != null) {
			return this.startTitle;
		}
		return super.getAppTitle();
	}
	
	@Override
	public String getAppName() {
		String appName = super.getAppName();
		if(count == 0) {
			return appName;
		} else {
			return appName.substring(0, appName.length()-1)+count+"/";
		}
	}
	
	@Override
	protected void addHeaders(IHttpRequest http, String uri)
			throws IOException {
		// TODO Auto-generated method stub
		super.addHeaders(http, uri);
	}
	
	@Override
	protected void close() {
		// TODO Auto-generated method stub
		super.close();
	}
}